/*jslint vars: true, plusplus: true, devel: true, nomen: true, regexp: true, indent: 4, maxerr: 50 */
/*global $, CSInterface */

(function () {
    'use strict';

    var cs = new CSInterface();
    var themeManager = new ThemeManager()

    cs.addEventListener('documentAfterSave', function(event) {
        var syncFile = false;
        cs.evalScript('callJSX('+syncFile+')', parseDataFormJSX)
    })

    var homePath = null
    cs.evalScript('getHomePath()', getHomePath);

    function getHomePath(payload) {
        homePath = payload
    }

    window.changeTheme = function () {
        var skinInfo = JSON.parse(window.__adobe_cep__.getHostEnvironment()).appSkinInfo
        themeManager.handleThemeChange(skinInfo)
    }

    window.documentChanged = function () {
        var syncFile = false;
        cs.evalScript('callJSX('+syncFile+')', parseDataFormJSX)
    }

    window.sync = function() {
        if(btn.classList.contains('disabled')){
            return
        }
        var syncFile = true;
        cs.evalScript('callJSX('+syncFile+')', parseDataFormJSX);
    }

    window.openDownloadsLink = function() {
        window.cep.util.openURLInDefaultBrowser("https://avocode.com/download")
    }

    var btn = document.getElementById('btn'),
        fileNameElm = document.getElementById('name'),
        fileSizeElm = document.getElementById('size');

    function readAvocodePath() {
        var appPathFilePath = homePath + '/.avocode/.app_path'
        var appPath = window.cep.fs.readFile(appPathFilePath, cep.encoding.UTF8)
        if (appPath.err !== 0) {
            return null
        }

        return appPath.data
    }

    function parseDataFormJSX(payload) {
        var SEPARATOR = '@@#DK#@@'
        var data = payload.split(SEPARATOR)
        var filePath = data[0]
        var error = data[1]
        var name = data[2]
        var fileSize = data[3]

        var dotParts = name.split('.')
        if (dotParts.length === 1) {
            // NOTE: File is missing extname, probably unsaved file in AI.
            error = 'document-is-not-saved'
        } else {
            var extname = dotParts[dotParts.length - 1].toLowerCase()
            var supportedExtnames = [ 'ai', 'psd', 'psb' ]
            var supportedExtname = (supportedExtnames.indexOf(extname) !== -1)

            var ai = false
            if (!supportedExtname && ai) {
                error = 'invalid-format-ai'
            } else if (!supportedExtname) {
                error = 'invalid-format'
            }
        }

        if (error != 'null') {
            fileSizeElm.innerHTML = ''
            btn.classList.add('disabled')
            if (error == 'document-is-not-saved') {
                return fileNameElm.innerHTML = 'You need to save this file before syncing to Avocode.'
            } else if (error == 'no-document-open') {
                return fileNameElm.innerHTML = 'Open a design first to sync it to Avocode.'
            } else if (error == 'invalid-format-ai') {
                return fileNameElm.innerHTML = 'Please save the design in PDF compatible AI format.'
            } else if (error == 'invalid-format') {
                return fileNameElm.innerHTML = 'This design format is not supported in Avocode.'
            }
            return fileNameElm.innerHTML = error;
        }

        btn.classList.remove('disabled')
        fileNameElm.innerHTML = name
        fileSizeElm.innerHTML = Math.round(fileSize/(1024*1024)*100)/100 + " MB"

        if (filePath && (filePath.indexOf('.avo') === (filePath.length - 4))) {
            var filePathBase64d = window.cep.encoding.convertion.utf8_to_b64(filePath)
            var fileUri = "file://" + filePath
            var syncUriBase = "avocode-sync://plugin-uploader/base64/"
            var syncUriBase64d = syncUriBase + filePathBase64d
            var appPath = readAvocodePath()
            if (!appPath) {
                fileSizeElm.innerHTML = ''
                btn.classList.add('disabled')
                fileNameElm.innerHTML = 'Avocode app not found.<br />Please <a id="downloadLink" href="#">install</a> it.'
                var downloadLink = document.getElementById("downloadLink")
                downloadLink.addEventListener("click", openDownloadsLink)

                return
            }

            window.cep.process.createProcess(appPath, '--', syncUriBase64d)
        }
    }

    btn.addEventListener("click", sync)
    window.__adobe_cep__ && window.__adobe_cep__.addEventListener('com.adobe.csxs.events.ThemeColorChanged', changeTheme)
    window.__adobe_cep__ && window.__adobe_cep__.addEventListener('documentAfterActivate', documentChanged)
    window.__adobe_cep__ && window.__adobe_cep__.addEventListener('documentAfterDeactivate', documentChanged)
    window.addEventListener('load', changeTheme, false)
    window.addEventListener('load', documentChanged, false)
}());
